import 'package:flutter/material.dart';
import 'package:widgetbook2/src/addons/widgets/addon_option.dart';
import 'package:widgetbook2/src/addons/widgets/addon_option_spacing.dart';

class AddonOptionList<T> extends StatelessWidget {
  const AddonOptionList({
    super.key,
    required this.name,
    required this.options,
    required this.selectedOptions,
    required this.onTap,
    required this.builder,
  });

  final String name;
  final List<T> options;
  final Set<T> selectedOptions;
  final void Function(T item) onTap;
  final Widget Function(T item) builder;

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.all(8),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(
            name,
            style: const TextStyle(
              fontSize: 20,
              fontWeight: FontWeight.bold,
            ),
          ),
          const SizedBox(
            height: 16,
          ),
          Expanded(
            child: ListView.separated(
              itemBuilder: (context, index) {
                final item = options[index];
                return AddonOption(
                  isSelected: selectedOptions.contains(item),
                  onTap: () {
                    onTap(item);
                  },
                  child: builder(item),
                );
              },
              separatorBuilder: (_, __) {
                return const AddonOptionSpacing();
              },
              itemCount: options.length,
            ),
          ),
        ],
      ),
    );
  }
}
